@@ -4,8 +4,11 @@
 #include "log.cpp"
 #endif
 
+static int KbdAnsi(char *Addr,int Size);
+
 #if !defined(GUI) && !defined(SILENT)
 static void RawPrint(char *Msg,MESSAGE_TYPE MessageType);
+static uint GetKey();
 #endif
 
 static MESSAGE_TYPE MsgStream=MSG_STDOUT;
@@ -64,8 +67,8 @@ void RawPrint(char *Msg,MESSAGE_TYPE MessageType)
     default:
       return;
   }
-#ifdef _WIN_32
-  CharToOem(Msg,Msg);
+#ifdef _WIN_ALL
+  CharToOemA(Msg,Msg);
 
   char OutMsg[MaxMsgSize],*OutPos=OutMsg;
   for (int I=0;Msg[I]!=0;I++)
@@ -87,7 +90,6 @@ void RawPrint(char *Msg,MESSAGE_TYPE MessageType)
 #endif
 
   OutFile.Write(Msg,strlen(Msg));
-//  OutFile.Flush();
 }
 #endif
 
@@ -105,9 +107,11 @@ void Alarm()
 
 #ifndef SILENT
 #ifndef GUI
-void GetPasswordText(char *Str,int MaxLength)
+void GetPasswordText(wchar *Str,uint MaxLength)
 {
-#ifdef _WIN_32
+  if (MaxLength==0)
+    return;
+#ifdef _WIN_ALL
   HANDLE hConIn=GetStdHandle(STD_INPUT_HANDLE);
   HANDLE hConOut=GetStdHandle(STD_OUTPUT_HANDLE);
   DWORD ConInMode,ConOutMode;
@@ -116,15 +120,19 @@ void GetPasswordText(char *Str,int MaxLength)
   GetConsoleMode(hConOut,&ConOutMode);
   SetConsoleMode(hConIn,ENABLE_LINE_INPUT);
   SetConsoleMode(hConOut,ENABLE_PROCESSED_OUTPUT|ENABLE_WRAP_AT_EOL_OUTPUT);
-  ReadConsole(hConIn,Str,MaxLength-1,&Read,NULL);
+
+  ReadConsoleW(hConIn,Str,MaxLength-1,&Read,NULL);
   Str[Read]=0;
-  OemToChar(Str,Str);
   SetConsoleMode(hConIn,ConInMode);
   SetConsoleMode(hConOut,ConOutMode);
-#elif defined(_EMX) || defined(_BEOS) || defined(__sparc) || defined(sparc) || defined (__VMS)
-  fgets(Str,MaxLength-1,stdin);
 #else
-  strncpyz(Str,getpass(""),MaxLength);
+  char StrA[MAXPASSWORD];
+#if defined(_EMX) || defined(_BEOS) || defined(__sparc) || defined(sparc) || defined (__VMS)
+  fgets(StrA,ASIZE(StrA)-1,stdin);
+#else
+  strncpyz(StrA,getpass(""),ASIZE(StrA));
+#endif
+  CharToWide(StrA,Str,MaxLength);
 #endif
   Str[MaxLength-1]=0;
   RemoveLF(Str);
@@ -133,29 +141,8 @@ void GetPasswordText(char *Str,int MaxLength)
 #endif
 
 
-#if !defined(GUI) && !defined(SILENT)
-unsigned int GetKey()
-{
-#ifdef SILENT
-  return(0);
-#else
-  char Str[80];
-#ifdef __GNUC__
-  fgets(Str,sizeof(Str),stdin);
-  return(Str[0]);
-#else
-  File SrcFile;
-  SrcFile.SetHandleType(FILE_HANDLESTD);
-  SrcFile.Read(Str,sizeof(Str));
-  return(Str[0]);
-#endif
-#endif
-}
-#endif
-
-
 #ifndef SILENT
-bool GetPassword(PASSWORD_TYPE Type,const char *FileName,char *Password,int MaxLength)
+bool GetPassword(PASSWORD_TYPE Type,const char *FileName,const wchar *FileNameW,wchar *Password,uint MaxLength)
 {
   Alarm();
   while (true)
@@ -180,12 +167,12 @@ bool GetPassword(PASSWORD_TYPE Type,const char *FileName,char *Password,int MaxL
     if (Type==PASSWORD_GLOBAL)
     {
       eprintf(St(MReAskPsw));
-      char CmpStr[MAXPASSWORD];
+      wchar CmpStr[MAXPASSWORD];
       GetPasswordText(CmpStr,ASIZE(CmpStr));
-      if (*CmpStr==0 || strcmp(Password,CmpStr)!=0)
+      if (*CmpStr==0 || wcscmp(Password,CmpStr)!=0)
       {
         eprintf(St(MNotMatchPsw));
-        memset(Password,0,MaxLength);
+        memset(Password,0,MaxLength*sizeof(*Password));
         memset(CmpStr,0,sizeof(CmpStr));
         continue;
       }
@@ -198,6 +185,29 @@ bool GetPassword(PASSWORD_TYPE Type,const char *FileName,char *Password,int MaxL
 #endif
 
 
+#if !defined(GUI) && !defined(SILENT)
+uint GetKey()
+{
+  char Str[80];
+  bool EndOfFile;
+#if defined(__GNUC__) || defined(sun)
+  EndOfFile=(fgets(Str,sizeof(Str),stdin)==NULL);
+#else
+  File SrcFile;
+  SrcFile.SetHandleType(FILE_HANDLESTD);
+  EndOfFile=(SrcFile.Read(Str,sizeof(Str))==0);
+#endif
+  if (EndOfFile)
+  {
+    // Looks like stdin is a null device. We can enter to infinite loop
+    // calling Ask(), so let's better exit.
+    ErrHandler.Exit(USER_BREAK);
+  }
+  return(Str[0]);
+}
+#endif
+
+
 #if !defined(GUI) && !defined(SILENT)
 int Ask(const char *AskStr)
 {
@@ -237,7 +247,7 @@ int Ask(const char *AskStr)
   }
   eprintf(" ");
   int Ch=GetKey();
-#if defined(_WIN_32)
+#if defined(_WIN_ALL)
   OemToCharBuff((LPCSTR)&Ch,(LPTSTR)&Ch,1);
 #endif
   Ch=loctoupper(Ch);
@@ -249,18 +259,18 @@ int Ask(const char *AskStr)
 #endif
 
 
-int KbdAnsi(char *Addr,int Size)
+int KbdAnsi(char *Addr,size_t Size)
 {
   int RetCode=0;
 #ifndef GUI
-  for (int I=0;I<Size;I++)
+  for (size_t I=0;I<Size;I++)
     if (Addr[I]==27 && Addr[I+1]=='[')
     {
-      for (int J=I+2;J<Size;J++)
+      for (size_t J=I+2;J<Size;J++)
       {
         if (Addr[J]=='\"')
           return(2);
-        if (!isdigit(Addr[J]) && Addr[J]!=';')
+        if (!IsDigit(Addr[J]) && Addr[J]!=';')
           break;
       }
       RetCode=1;
@@ -270,16 +280,16 @@ int KbdAnsi(char *Addr,int Size)
 }
 
 
-void OutComment(char *Comment,int Size)
+void OutComment(char *Comment,size_t Size)
 {
 #ifndef GUI
   if (KbdAnsi(Comment,Size)==2)
     return;
-  const int MaxOutSize=0x400;
-  for (int I=0;I<Size;I+=MaxOutSize)
+  const size_t MaxOutSize=0x400;
+  for (size_t I=0;I<Size;I+=MaxOutSize)
   {
     char Msg[MaxOutSize+1];
-    int CopySize=Min(MaxOutSize,Size-I);
+    size_t CopySize=Min(MaxOutSize,Size-I);
     strncpy(Msg,Comment+I,CopySize);
     Msg[CopySize]=0;
     mprintf("%s",Msg);
